/*
 * Copyright (c) 2005-2013 Dirk Aporius <dirk.aporius@gmail.com>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package myTreasure.game;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.event.KeyEvent;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;

import myTreasure.MyTreasureConstants;
import myTreasure.userlevels.MyTreasureLevels;

public class MyTreasureGame extends MyTreasureModel {

	public static final String BACK = "back";
	public static final String INGAME_RESTART = "ingame_restart";
	public static final String INGAME_NEXT = "ingame_next";
	public static final String INGAME_PREVIOUS = "ingame_previous";
	public static final String RESULT_RESTART = "result_restart";
	public static final String RESULT_NEXT = "result_next";
	public static final String RESULT_MENU = "result_menu";
	
	private boolean bUserLevel;
	
	private boolean left, right, n, p, r, enter, mouseReleased;
	private int pressedX;
	
	private int state = 0;
	private int turn = 0;
	private int breath = 0;

	private int gold = 0;
	private int silver = 0;
	private int steps = 0;
	private int curLevel = 0;
	private int max = 1;
	/**
	 * 0 = from up to down
	 * 1 = from right to left
	 * 2 = from down to up
	 * 3 = from left to right
	 */
	private int gravity = 0;
	private int[][] level = new int[12][6];
	private int[][] levelBlocks = new int[12][6];
	private final int[][] lights = new int[16][10];
	private boolean bChange = false;
	private boolean bFall = true;
	private boolean bPlayerStop = false;
	private boolean bPlayerFall = false;
	private boolean bEditor = false;
	private boolean bStart = false;
	
	private String levelString = "";
	private String editorString = "";
	private String idleString = "";
	
	private int idleTime = 0;
	
	private BufferedImage light, screenBackground;
	
	public MyTreasureGame(final MyTreasurePanel game) {
		super(game);
	}

	@Override
	public void init() {
		this.lights[0][0] = 300;
		this.lights[0][1] = 700;
		this.lights[0][2] = 500;
		this.level[0][0] = -1;

		if (this.screenBackground == null) {
			this.makeScreenBackground();
		}
		if (this.light == null) {
			this.light = new BufferedImage(64, 64, BufferedImage.TYPE_INT_ARGB_PRE);
			Graphics2D g = this.light.createGraphics();
			g.setColor(new Color(255,200,0,4));
			for (int i = 5; i < 30; i += 1) {
				g.fillOval(0 + i, 0 + i, 64 - i*2, 64 - i*2);
			}
			g.dispose();
		}
	}

	public void makeScreenBackground() {
		this.screenBackground = new BufferedImage(320,480,BufferedImage.TYPE_INT_RGB);
		Graphics2D g = screenBackground.createGraphics();
		for (int y = 0; y < 16; y += 1) {
			for (int x = 0; x < 11; x += 1) {
				g.drawImage(this.getGame().getISheet().getSubimage((int)(Math.random() * 4) * 32, 3 * 32, 32, 32), (x-1) * 32, (y) * 32, null);
			}				
			if (y == 14) {
				g.drawImage(this.getGame().getISheet().getSubimage(0, 128, 192, 32), 0, 0, null);
				g.drawImage(this.getGame().getISheet().getSubimage(128, 64, 64, 32), 192, 0, null);
				g.drawImage(this.getGame().getISheet().getSubimage(128, 96, 64, 32), 256, 0, null);

				g.drawImage(this.getGame().getISheet().getSubimage(128, 64, 64, 32), 0, 448, null);
				g.drawImage(this.getGame().getISheet().getSubimage(128, 96, 63, 32), 64, 448, null);
				g.drawImage(this.getGame().getISheet().getSubimage(132, 64, 60, 32), 127, 448, null);
				g.drawImage(this.getGame().getISheet().getSubimage(128, 96, 63, 32), 187, 448, null);
				g.drawImage(this.getGame().getISheet().getSubimage(132, 64, 6, 32), 250, 448, null);
				g.drawImage(this.getGame().getISheet().getSubimage(128, 96, 64, 32), 256, 448, null);
			}
		}
		g.dispose();
	}
	
	public void setUserLevel(boolean bUserLevel) {
		this.bUserLevel = bUserLevel;
	}
	
	public void loadLevel(final boolean bEditor, final int levelInt, String levelString) {
		this.bEditor = bEditor;
		
		int level = levelInt;
		if (this.bEditor) {
			level = 0;
			if (levelString.length() > 0) {
				this.editorString = levelString;
			} else {
				levelString = this.editorString;
			}
		} else if ((levelString == null) || (levelString.length() <= 0)) {
			if (this.bUserLevel) {
				if ((MyTreasureLevels.editorLevels != null) && (MyTreasureLevels.editorLevels.length > 0)) {
					if (level < 0) {
						level = MyTreasureLevels.editorLevels.length - 1;
					}
					if (level >= MyTreasureLevels.editorLevels.length) {
						level = 0;
					}
					levelString = MyTreasureLevels.editorLevels[level];
				} else {
					level = 0;
					levelString = MyTreasureLevels.getLevel(level);
				}
			} else {
				this.max = this.getGame().getCurMax();
				int start = this.getGame().getCurStart();
				if (level < 0) {
					level = this.max - 1;
				}
				if (level >= this.max) {
					level = 0;
				}
				levelString = MyTreasureLevels.getLevel(level + start);
			}
		}
		this.levelString = levelString;
		
		this.changeState(0);
		this.curLevel = level;
		this.n = this.p = this.r = this.left = this.right = false;
		this.gravity = 0;
		this.steps = 0;
		this.bFall = true;
		this.bChange = false;
		this.turn = 0;
		this.idleTime = 0;
		this.idleString = "";
		this.gold = Integer.valueOf(levelString.substring(levelString.length() - 4, levelString.length() - 2));
		this.silver = Integer.valueOf(levelString.substring(levelString.length() - 2, levelString.length()));
		int i = Integer.valueOf(levelString.substring(0, 2));
		int j = Integer.valueOf(levelString.substring(2, 4));
		
		this.level = new int[j*2][i];
		this.levelBlocks = new int[j*2][i];
		
		for (int y = 0; y < this.level.length/2; y += 1) {
			for (int x = 0; x < this.level[0].length; x += 1) {
				this.levelBlocks[y][x] = 0;
				this.level[y][x] = Integer.valueOf(levelString.substring(y * this.level[0].length + x + 4, y * this.level[0].length + x + 5));
				if (this.level[y][x] == 0) {
					this.level[y+this.level.length/2][x] = (int)(Math.random()*4);
				}
			}
		}
		
		for (int y = 1; y < 14; y += 1) {
			for (int x = 0; x < 10; x += 1) {
				this.lights[y][x] = 0;
				if ((y < 4) || (y > 10)) {
					if ((this.curLevel == 0) && (this.getGame().getDifficulty() == MyTreasureLevelChooser.EASY)) {
						
					} else if (Math.random() * 100 < 3) {
						this.lights[y][x] = (int)(Math.random() * 900) + 20;
					}
				}
			}
		}
	}

	public void keyPressed(int keyCode, char keyCharacter) {
		if (keyCode == KeyEvent.VK_LEFT) {
			this.left = true;
		}
		if (keyCode == KeyEvent.VK_RIGHT) {
			this.right = true;
		}
	}
	
	@Override
	public void keyButtonReleased(int keyCode, char character) {
		if (keyCode == KeyEvent.VK_LEFT) {
			this.left = false;
		}
		if (keyCode == KeyEvent.VK_RIGHT) {
			this.right = false;
		}
		if (keyCode == KeyEvent.VK_N) {
			this.n = true;
		}
		if (keyCode == KeyEvent.VK_P) {
			this.p = true;
		}
		if (keyCode == KeyEvent.VK_R) {
			this.r = true;
		}
		if (keyCode == KeyEvent.VK_S) {
			if ((this.levelString != null) && (this.levelString.length() > 0)) {
				this.getGame().solveLevel(this.levelString);
			}
		}

		if (keyCode == KeyEvent.VK_ENTER) {
			this.enter = true;
		}
		if (this.bStart) {
			this.bStart = false;
		}
	}

	@Override
	public void mouseButtonFunction(String function) {
		if ((function.equals(MyTreasureGame.BACK)) || (function.equals(MyTreasureGame.RESULT_MENU))) {
			if (this.bEditor) {
				this.getGame().setEditor(false, 95);
			} else if (this.bUserLevel) {
				this.getGame().setMenu();
			} else {
				this.getGame().setLevelChooser(this.getGame().getDifficulty(), this.curLevel);
			}
		} else if ((function.equals(MyTreasureGame.INGAME_RESTART)) || (function.equals(MyTreasureGame.RESULT_RESTART))) {
			this.restartLevel();
		} else if ((function.equals(MyTreasureGame.INGAME_NEXT)) || (function.equals(MyTreasureGame.RESULT_NEXT))) {
			this.loadNextLevel();
		} else if (function.equals(MyTreasureGame.INGAME_PREVIOUS)) {
			this.loadPreviousLevel();
		}
	}

	@Override
	public void mouseButtonReleased(int x, int y, boolean bRight) {
		if ((this.turn == 0) && (this.state == 0)) {
			if (this.pressedX - x < -MyTreasureConstants.CHANGE_DRAG) {
				if (y < MyTreasureConstants.GAME_HEIGHT/2) {
					this.rotateRight();
				} else {
					this.rotateLeft();
				}
			}
			if (this.pressedX - x > MyTreasureConstants.CHANGE_DRAG) {
				if (y < MyTreasureConstants.GAME_HEIGHT/2) {
					this.rotateLeft();
				} else {
					this.rotateRight();
				}
			}
		}
		this.mouseReleased = true;
	}

	public boolean mouseMoved(int x, int y) {
		return false;
	}
	
	public boolean mouseDragged(int x, int y, boolean bRight) {
		return false;
	}
	
	public boolean mousePressed(int x, int y, boolean bRight) {
		this.pressedX = x;
		return false;
	}
	
	@Override
	public void think(int delta) {
		for (int y = 0; y < 15; y += 1) {
			for (int x = 0; x < 10; x += 1) {
				if (this.lights[y][x] > 0) {
					this.lights[y][x] -= 10;
					if (this.lights[y][x] <= 0) {
						this.lights[y][x] = (int)(Math.random() * 100 + 850);
					}
				}
			}
		}
		
		if (this.state == 0) {
			if (this.breath > 0) {
				this.breath -= delta;
			}
			
			if (this.level[0][0] == -1) {
				this.loadLevel(this.bEditor, this.curLevel, "");
			} else if (this.turn == 0) {
				if (this.bFall) {
					if (!this.bChange) {
						int i = 0;
						int j = 1;
						if (this.gravity == 0) {
							for (int y = this.level.length/2 - 2; y > 0; y -= 1) {
								for (int x = 1; x < this.level[0].length - 1; x += 1) {
									if (this.level[y][x] == 2) {
										if (((this.level[y+j][x+i] == 1) || (this.level[y+j][x+i] == 3)) && (this.level[y+j][x+i] < 4)) {
											this.level[y][x] = 1;
											this.level[y+j][x+i] = 2;
											if (this.level[y+j][x+i] == 3) {
												this.level[y+j][x+i] = 0;
											}
											this.bPlayerFall = true;
											this.bChange = true;
											this.levelBlocks[y+j][x+i] = -32;
										} else if (bPlayerStop) {
											this.bPlayerStop = false;
											this.breath = 100;
										}
									}
									if (this.level[y][x] == 4) {
										if (this.level[y+j][x+i] == 1) {
											this.level[y][x] = 1;
											this.level[y+j][x+i] = 4;

											this.bChange = true;
											this.levelBlocks[y+j][x+i] = -32;

										}
									}
								}
							}
						}
						if (this.gravity == 2) {
							j = -1;
							for (int y = 1; y < this.level.length/2 - 1; y += 1) {
								for (int x = 1; x < this.level[0].length - 1; x += 1) {
									if (this.level[y][x] == 2) {
										if (((this.level[y+j][x+i] == 1) || (this.level[y+j][x+i] == 3)) && (this.level[y+j][x+i] < 4)) {
											this.level[y][x] = 1;
											this.level[y+j][x+i] = 2;
											if (this.level[y+j][x+i] == 3) {
												this.level[y+j][x+i] = 0;
											}
											this.bPlayerFall = true;
											this.bChange = true;
											this.levelBlocks[y+j][x+i] = 32;
										} else if (this.bPlayerStop) {
											this.bPlayerStop = false;
											this.breath = 100;
										}
									}
									if (this.level[y][x] == 4) {
										if (this.level[y+j][x+i] == 1) {
											this.level[y][x] = 1;
											this.level[y+j][x+i] = 4;
											
											this.bChange = true;
											this.levelBlocks[y+j][x+i] = 32;
										}
									}
								}
							}
						}
						if (this.gravity == 1) {
							i = 1;
							j = 0;
							for (int y = this.level.length/2 - 2; y > 0; y -= 1) {
								for (int x = this.level[0].length - 2; x > 0; x -= 1) {
									if (this.level[y][x] == 2) {
										if (((this.level[y+j][x+i] == 1) || (this.level[y+j][x+i] == 3)) && (this.level[y+j][x+i] < 4)) {
											this.level[y][x] = 1;
											this.level[y+j][x+i] = 2;
											if (this.level[y+j][x+i] == 3) {
												this.level[y+j][x+i] = 0;
											}
											this.bPlayerFall = true;
											this.bChange = true;
											this.levelBlocks[y+j+this.levelBlocks.length/2][x+i] = -32;
										} else if (this.bPlayerStop) {
											this.bPlayerStop = false;
											this.breath = 100;
										}
									}
									if (this.level[y][x] == 4) {
										if (this.level[y][x+i] == 1) {
											this.level[y][x] = 1;
											this.level[y][x+i] = 4;
											
											this.bChange = true;
											this.levelBlocks[y+this.levelBlocks.length/2][x+i] = -32;
										}
									}
								}
							}
						}
						if (this.gravity == 3) {
							i = -1;
							j = 0;
							for (int y = this.level.length/2 - 2; y > 0; y -= 1) {
								for (int x = 1; x < this.level[0].length - 1; x += 1) {
									if (this.level[y][x] == 2) {
										if (((this.level[y+j][x+i] == 1) || (this.level[y+j][x+i] == 3)) && (this.level[y+j][x+i] < 4)) {
											this.level[y][x] = 1;
											this.level[y+j][x+i] = 2;
											if (this.level[y+j][x+i] == 3) {
												this.level[y+j][x+i] = 0;
											}
											this.bPlayerFall = true;
											this.bChange = true;
											this.levelBlocks[y+j+this.levelBlocks.length/2][x+i] = 32;
										} else if (this.bPlayerStop) {
											this.bPlayerStop = false;
											this.breath = 100;
										}
									}
									if (this.level[y][x] == 4) {
										if (this.level[y][x+i] == 1) {
											this.level[y][x] = 1;
											this.level[y][x+i] = 4;
											
											this.bChange = true;
											this.levelBlocks[y+this.levelBlocks.length/2][x+i] = 32;
										}
									}
								}
							}
						}
						if (!this.bChange) {
							this.bFall = false;
							boolean bWin = true;
							for (int y = 1; y < this.level.length/2; y += 1) {
								for (int x = 1; x < this.level[0].length; x += 1) {
									if (this.level[y][x] == 3) {
										bWin = false;
										break;
									}
								}
							}
							if (bWin) {
								if (this.bEditor) {
									int step = this.steps;
									if (step >= 100) {
										step = 95;
									}
									this.getGame().setEditor(true, step);
								} else {
									this.changeState(1);
									this.saveSolvedLevel();
								}
							}
						}
					} else {
						for (int y = this.level.length/2 - 1; y >= 0; y -= 1) {
							for (int x = 0; x < this.level[0].length; x += 1) {
								if (this.levelBlocks[y][x] < 0) {
									this.levelBlocks[y][x] += 2f;
									if (this.levelBlocks[y][x] >= 0) this.levelBlocks[y][x] = 0;
									
									if (this.level[y][x] == 2) {
										if (this.levelBlocks[y][x] == 0)	{
											this.bPlayerStop = true;
											this.bPlayerFall = false;
										} else {
											this.bPlayerFall = true;
										}
									}
								}
								if (this.levelBlocks[y][x] > 0) {
									this.levelBlocks[y][x] -= 2f;
									if (this.levelBlocks[y][x] <= 0) this.levelBlocks[y][x] = 0;
									
									if (this.level[y][x] == 2) {
										if (this.levelBlocks[y][x] == 0)	{
											this.bPlayerStop = true;
											this.bPlayerFall = false;
										} else {
											this.bPlayerFall = true;
										}
									}
								}
								if (this.levelBlocks[y+this.levelBlocks.length/2][x] < 0) {
									this.levelBlocks[y+this.levelBlocks.length/2][x] += 2f;
									if (this.levelBlocks[y+this.levelBlocks.length/2][x] >= 0) this.levelBlocks[y+this.levelBlocks.length/2][x] = 0;
									
									if (this.level[y][x] == 2) {
										if (this.levelBlocks[y+this.levelBlocks.length/2][x] == 0)	{
											this.bPlayerStop = true;
											this.bPlayerFall = false;
										} else {
											this.bPlayerFall = true;
										}
									}
								}
								if (this.levelBlocks[y+this.levelBlocks.length/2][x] > 0) {
									this.levelBlocks[y+this.levelBlocks.length/2][x] -= 2f;
									if (this.levelBlocks[y+this.levelBlocks.length/2][x] <= 0) this.levelBlocks[y+this.levelBlocks.length/2][x] = 0;
									
									if (this.level[y][x] == 2) {
										if (this.levelBlocks[y+this.levelBlocks.length/2][x] == 0)	{
											this.bPlayerStop = true;
											this.bPlayerFall = false;
										} else {
											this.bPlayerFall = true;
										}
									}
								}
							}
						}
						this.bChange = false;
						for (int y = 0; y < this.level.length/2; y += 1) {
							for (int x = 0; x < this.level[0].length; x += 1) {
								if ((this.levelBlocks[y][x] != 0) || (this.levelBlocks[y+this.levelBlocks.length/2][x] != 0)) {
									this.bChange = true;
								}										
							}
						}
						if (!this.bChange) {
							this.think(delta);
						}
					}
				} else if (!this.bStart) {
					this.idleTime += delta;
					if ((this.idleTime - delta < MyTreasureConstants.START_IDLE_TEXT_TIME) && (this.idleTime >= MyTreasureConstants.START_IDLE_TEXT_TIME)) {
						this.idleString = MyTreasureConstants.TEXT_IDLE[(int)(Math.random() * MyTreasureConstants.TEXT_IDLE.length)];
					}
					if ((this.idleTime - delta < MyTreasureConstants.END_IDLE_TEXT_TIME) && (this.idleTime >= MyTreasureConstants.END_IDLE_TEXT_TIME)) {
						this.idleString = "";
						this.idleTime = 0;
					}
					
					if (this.left) {
						this.rotateLeft();
					}
					if (this.right) {
						this.rotateRight();
					}
					if (this.n) {
						this.loadNextLevel();
					}
					if (this.r) {
						this.restartLevel();
					}
					if (this.p) {
						this.loadPreviousLevel();
					}
				}
			} else {
				if (this.turn < 0) {
					this.turn += 5;
					if (this.turn >= 0) {
						this.turn = 0;
					}
				}
				if (this.turn > 0) {
					this.turn -= 5;
					if (this.turn <= 0) {
						this.turn = 0;
					}
				}
			}
		}
		if (this.state == 1) {
			if ((this.enter) || (this.mouseReleased) || (this.right) || (this.left)) {
				if ((this.left) || (this.right)) {
					this.bStart = true;
				}
				this.changeState(0);
				this.loadNextLevel();
			}
			if (this.r) {
				this.restartLevel();
			}
		}
		
		this.mouseReleased = false;
		this.enter = this.n = this.p = this.r = false;
	}
	
	private void saveSolvedLevel() {
		if (!this.bEditor) {
			int change = 0;
			int skulls = 1;
			if (this.steps <= this.gold) {
				skulls = 3;
			} else if (this.steps <= this.silver) {
				skulls = 2;
			}
			if (this.getGame().getSolvedLevel().containsKey(this.levelString)) {
				if (skulls > this.getGame().getSolvedLevel().get(this.levelString)) {
					change = skulls - this.getGame().getSolvedLevel().get(this.levelString);
					this.getGame().getSolvedLevel().remove(this.levelString);
					this.getGame().getSolvedLevel().put(this.levelString, skulls);
				}
			} else {
				this.getGame().getSolvedLevel().put(this.levelString, skulls);
				change = skulls;
			}
			if ((!this.bUserLevel) && (change > 0)) {
				this.getGame().setCurSkulls(this.getGame().getCurSkulls() + change);
			}
		}
	}
	
	private void rotateLeft() {
		this.idleTime = 0;
		this.idleString = "";
		this.bFall = true;
		this.gravity -= 1;
		if (this.gravity < 0) this.gravity = 3;
		this.turn = -90;
		this.steps += 1;
	}
	
	private void rotateRight() {
		this.idleTime = 0;
		this.idleString = "";
		this.bFall = true;
		this.gravity += 1;
		if (this.gravity > 3) this.gravity = 0;
		this.turn = 90;
		this.steps += 1;
	}
	
	private void changeState(final int state) {
		if (this.state != state) {
			this.state = state;
			boolean bVisible = true;
			if (this.state == 1) {
				bVisible = false;
			}
			this.getGame().getButtons()[6].setBVisible(bVisible);
			this.getGame().getButtons()[8].setBVisible(bVisible);
			this.getGame().getButtons()[9].setBVisible(bVisible);
			this.getGame().getButtons()[10].setBVisible(bVisible);
			
			this.getGame().getButtons()[11].setBVisible(!bVisible);
			this.getGame().getButtons()[12].setBVisible(!bVisible);
			this.getGame().getButtons()[13].setBVisible(!bVisible);
		}
	}
	
	private void loadNextLevel() {
		this.curLevel += 1;
		this.level[0][0] = -1;
		this.changeState(0);
	}
	
	private void loadPreviousLevel() {
		this.curLevel -= 1;
		this.level[0][0] = -1;
		this.changeState(0);
	}
	
	private void restartLevel() {
		this.level[0][0] = -1;
		this.changeState(0);
	}

	@Override
	public void render(Graphics2D g) {
		AffineTransform oldAffineTransform = g.getTransform();
		// Renderabschnitt
		// Hintergrund malen
		g.drawImage(this.screenBackground, 0, 0, null);
		
		for (int y = 1; y < this.lights.length; y += 1) {
			for (int x = 0; x < this.lights[0].length; x += 1) {
				if (this.lights[y][x] > 0) {
					g.drawImage(this.light, (x) * 32 - 16, (y) * 32 - 16, null);
					if (this.lights[y][x] > 450) {
						g.drawImage(this.getGame().getISheet().getSubimage(1 * 32, 0 * 32, 32, 32), (x) * 32, (y) * 32, null);
					} else {
						g.drawImage(this.getGame().getISheet().getSubimage(3 * 32, 1 * 32, 32, 32), (x) * 32, (y) * 32, null);
					}
				}
			}
		}
		
		if (this.state == 0) {
			g.setTransform(AffineTransform.getRotateInstance(Math.toRadians(90 * this.gravity - this.turn), 160, 240));
			int i = MyTreasureConstants.GAME_WIDTH/2 - this.level[0].length * 16;
			int j = MyTreasureConstants.GAME_HEIGHT/2 - this.level.length/2 * 16;
			for (int y = 0; y < this.level.length/2; y += 1) {
				for (int x = 0; x < this.level[0].length; x += 1) {
					if (this.level[y][x] == 3) {
						g.drawImage(this.light, i + (x) * 32 - 16, j+(y) * 32 - 16, null);
						g.drawImage(this.getGame().getISheet().getSubimage(0 * 32, 1 * 32, 32, 32), i + (x) * 32, j + (y) * 32, null);
					}
					if (this.level[y][x] == 4) {
						g.drawImage(this.getGame().getISheet().getSubimage(2 * 32, 0 * 32, 32, 32), i + (x) * 32 + (int)(this.levelBlocks[y+this.levelBlocks.length/2][x]), j + (y) * 32 + (int)(this.levelBlocks[y][x]), null);
					}
					if (this.level[y][x] == 0) {
						g.drawImage(this.getGame().getISheet().getSubimage(this.level[y + this.level.length/2][x] * 32, 2 * 32, 32, 32), i + (x) * 32 + (int)(this.levelBlocks[y+this.levelBlocks.length/2][x]), j + (y) * 32 + (int)(this.levelBlocks[y][x]), null);
					}
					if (this.level[y][x] == 2) {
						int curX = 0;
						int curY = 0;
						if (this.bPlayerFall) {
							curX = 3;
						}
						if (this.breath > 0) {
							curX = 2;
							curY = 1;
						}
						if (this.gravity%2!=0) {
							curY = 0;
							curX = 4;
							if (this.bPlayerFall) {
								curX = 6;
							}
							if (this.breath > 0) {
								curX = 5;
							}
						}
						if ((this.gravity == 0) || (this.gravity == 3)) {
							g.drawImage(this.getGame().getISheet().getSubimage(curX * 32, curY * 32, 32, 32), i + (x) * 32 + (int)(this.levelBlocks[y+this.levelBlocks.length/2][x]), j + (y) * 32 + (int)(this.levelBlocks[y][x]), null);	
						}
						if ((this.gravity == 2) || (this.gravity == 1)) {
							g.drawImage(this.getGame().getISheet().getSubimage(curX * 32, curY * 32, 32, 32), i + (x) * 32 + (int)(this.levelBlocks[y+this.levelBlocks.length/2][x]), j + (y) * 32 + (int)(this.levelBlocks[y][x]), i + (x + 1) * 32 + (int)(this.levelBlocks[y+this.levelBlocks.length/2][x]), j + (y+1) *32 + (int)(this.levelBlocks[y][x]), 32, 32, 0, 0, null);	
						}
						
						if (this.idleString.length() > 0) {
							g.setFont(MyTreasureConstants.fontMedium);
							int w = g.getFontMetrics().stringWidth(this.idleString);
							g.setColor(Color.WHITE);
							g.fillRect(i + (x) * 32 + (int)(this.levelBlocks[y+this.levelBlocks.length/2][x]) + 16 - w/2 - 4, j + (y) * 32 + (int)(this.levelBlocks[y][x]) - 29, w + 8, 28);
							g.setColor(new Color(68, 68, 68));
							g.drawString(this.idleString, i + (x) * 32 + (int)(this.levelBlocks[y+this.levelBlocks.length/2][x]) + 16 - w/2, j + (y) * 32 + (int)(this.levelBlocks[y][x]) - 9);
						}
					}
				}
			}
			g.setTransform(oldAffineTransform);
		}
		
		String s = "";
		int w = 0;

		g.setFont(MyTreasureConstants.fontBig);
		if (this.state == 1) {
			g.setColor(new Color(175,175,175));
			
			s = "Congratulation";
			w = g.getFontMetrics().stringWidth(s);
			g.drawString(s, 160 - w/2, 96);
			
			g.drawImage(this.light, 20, 234, null);
			if (this.lights[0][1] > 450) {
				g.drawImage(this.getGame().getISheet().getSubimage(1 * 32, 0 * 32, 32, 32), 36, 250, null);
			} else {
				g.drawImage(this.getGame().getISheet().getSubimage(3 * 32, 1 * 32, 32, 32), 36, 250, null);
			}
			
			g.drawImage(this.light, 236, 234, null);
			if (this.lights[0][0] > 450) {
				g.drawImage(this.getGame().getISheet().getSubimage(1 * 32, 0 * 32, 32, 32), 252, 250, null);
			} else {
				g.drawImage(this.getGame().getISheet().getSubimage(3 * 32, 1 * 32, 32, 32), 252, 250, null);
			}
			
			if (this.lights[0][2] > 450) {
				g.drawImage(this.getGame().getISheet().getSubimage(4 * 32, 1 * 32, 32, 32), 144, 150, null);
			} else {
				g.drawImage(this.getGame().getISheet().getSubimage(5 * 32, 1 * 32, 32, 32), 144, 150, null);
			}
			g.drawImage(this.getGame().getISheet().getSubimage(2 * 32, 0 * 32, 32, 32), 80, 150, null);
			g.drawImage(this.getGame().getISheet().getSubimage(2 * 32, 0 * 32, 32, 32), 208, 150, null);
			
			g.setFont(MyTreasureConstants.fontMedium);

			s = "You needed";
			w = g.getFontMetrics().stringWidth(s);
			g.drawString(s, 160 - w/2, 244);
			
			s = "moves.";
			w = g.getFontMetrics().stringWidth(s);
			g.drawString(s, 160 - w/2, 304);
			
			if (this.steps <= this.gold) {
				g.setColor(new Color(255, 235, 0));
			}
			if (this.steps > this.silver) {
				g.setColor(new Color(215, 100, 50));
			}
			s = String.valueOf(this.steps);
			w = g.getFontMetrics().stringWidth(s);
			g.drawString(s, 160 - w/2, 274);
			
			if (this.steps <= this.gold) {
				g.drawImage(this.light, 108 - 16, 332 - 16, null);
				g.drawImage(this.getGame().getISheet().getSubimage(0 * 32, 1 * 32, 32, 32), 108, 332, null);
				g.drawImage(this.light, 144 - 16, 332 - 16, null);
				g.drawImage(this.getGame().getISheet().getSubimage(0 * 32, 1 * 32, 32, 32), 144, 332, null);
				g.drawImage(this.light, 180 - 16, 332 - 16, null);
				g.drawImage(this.getGame().getISheet().getSubimage(0 * 32, 1 * 32, 32, 32), 180, 332, null);
			} else if (this.steps <= this.silver) {
				g.drawImage(this.light, 126 - 16, 332 - 16, null);
				g.drawImage(this.getGame().getISheet().getSubimage(6 * 32, 1 * 32, 32, 32), 126, 332, null);
				g.drawImage(this.light, 162 - 16, 332 - 16, null);
				g.drawImage(this.getGame().getISheet().getSubimage(6 * 32, 1 * 32, 32, 32), 162, 332, null);
			} else {
				g.drawImage(this.light, 144 - 16, 332 - 16, null);
				g.drawImage(this.getGame().getISheet().getSubimage(1 * 32, 1 * 32, 32, 32), 144, 332, null);
			}
		} else if ((!this.bEditor) && (!this.bUserLevel)) {
			if ((this.curLevel == 0) && (this.getGame().getDifficulty() == MyTreasureLevelChooser.EASY)) {
				g.setColor(new Color(175,175,175));
				g.setFont(MyTreasureConstants.fontMedium);
				for (int i = 0; i < MyTreasureConstants.FIRST_COLLECT.length; i++) {
					s = MyTreasureConstants.FIRST_COLLECT[i];
					w = g.getFontMetrics().stringWidth(s);
					g.drawString(s, MyTreasureConstants.GAME_WIDTH/2 - w/2, 65 + 40 * i);
				}
				
				g.setFont(MyTreasureConstants.fontSmall);
				for (int i = 0; i < MyTreasureConstants.FIRST_MOVE.length; i++) {
					s = MyTreasureConstants.FIRST_MOVE[i];
					w = g.getFontMetrics().stringWidth(s);
					g.drawString(s, MyTreasureConstants.GAME_WIDTH/2 - w/2, 355 + 22 * i);
				}
			}
		}
		g.setFont(MyTreasureConstants.fontSmall);
		g.setColor(new Color(50,50,50));
		
		if (this.bUserLevel) {
			s = "Level "+String.valueOf(curLevel+1)+"/"+MyTreasureLevels.editorLevels.length;
		} else {
			s = "Level "+String.valueOf(curLevel+1)+"/"+this.max;
		}
		w = g.getFontMetrics().stringWidth(s);

		if ((!this.bEditor) && (this.getGame().getSolvedLevel().containsKey(this.levelString))) {
			g.drawImage(this.getGame().getISheet().getSubimage(8 * 4 + 16 * (this.getGame().getSolvedLevel().get(this.levelString) - 1), 120 * 4, 12, 12), 160 + w/2 + 2, 458, null);
		}
		
		g.drawString(s, 160 - w/2, 470);

		if (this.silver > 0) {
			w = (int)((this.steps) * 100 / (this.silver));
			if (w > 98) {
				w = 98;
			}
			if (w < 0) {
				w = 0;
			}
			
			if (this.steps <= this.gold) {
				// gold
				g.setColor(new Color(255, 235, 50));
				g.fillRect(4, 8, (int)(185 - w * 1.84f), 20);
				g.setColor(new Color(255, 255, 200));
				g.fillRect(4, 12, (int)(185 - w * 1.84f), 4);
				g.setColor(new Color(215, 150, 50));
				g.fillRect(4, 24, (int)(185 - w * 1.84f), 4);
			} else if (this.steps > this.silver) {
				// bronze
				g.setColor(new Color(215, 100, 50));
				g.fillRect(4, 8, (int)(185 - w * 1.84f), 20);
				g.setColor(new Color(235, 185, 150));
				g.fillRect(4, 12, (int)(185 - w * 1.84f), 4);
				g.setColor(new Color(185, 90, 40));
				g.fillRect(4, 24, (int)(185 - w * 1.84f), 4);
			} else {
				// silver
				g.setColor(new Color(150, 150, 150));
				g.fillRect(4, 8, (int)(185 - w * 1.84f), 20);
				g.setColor(new Color(235, 235, 235));
				g.fillRect(4, 12, (int)(185 - w * 1.84f), 4);
				g.setColor(new Color(135, 135, 135));
				g.fillRect(4, 24, (int)(185 - w * 1.84f), 4);
			}
			g.setColor(MyTreasureConstants.COLOR_SEPARATOR);
			g.fillRect((int)(185 - 96 * 1.85f), 8, 2, 20);
			
			int si = (int)((this.gold) * 100 / (this.silver));
			g.fillRect((int)(185 - (si) * 1.85f), 8, 2, 20);
		}
	}

}
