/*
 * Copyright (c) 2005-2013 Dirk Aporius <dirk.aporius@gmail.com>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.apogames;

import java.awt.Graphics2D;

import org.apogames.input.ApoKeyboard;
import org.apogames.input.ApoMouse;

/**
 * abstrakte Klasse von der das eigentliche Spielpanel erbt und alles wichtige handelt
 * von den Keyboardeingaben, Mauseingaben und die "eigentliche Engine" bereitstellt
 * 
 * @author Dirk Aporius
 *
 */
public abstract class ApoSubGame extends Thread {

	/** das Screenobjekt fr das Spiel */
	protected final ApoScreen screen;
	/** das Keyboardeingabeobjekt fr das Spiel */
	protected ApoKeyboard keyboard;
	/** das Mausobjekt fr das Spiel */
	protected ApoMouse mouse;
	/** Integer Variable welche angibt, wie die nchste SubGame ID ist */
	private int nextID = 0;
	/** boolean Variable welche angibt, ob ein Spiel gerade luft oder nicht */
	private boolean isRunning = false;
	/** long Vaitbale die angibt, welche Zeit beim letzten Aufruf war in ms */
	private long last;
	/** Zhlt die Anzahl der Neuzeichnungen pro Sekunde */
	private int frames;
	/** boolean Variable die angibt, ob das Spiel immer nachdenken soll */
	private boolean bShouldThink;
	/** boolean Variable die angibt, ob das Spiel immer neu gezeichnet werden soll */
	private boolean bShouldRepaint;
	/** long Variable die angibt, wie lange das Spiel beim letzten Zeichnen bentigt hat in ns */
	protected long renderTime;
	/** long Variable die angibt, wie lange das Spiel beim letzten Nachdenken bentigt hat in ns */
	private long thinkTime;

	/**
	 * Konstruktor mit dem eigentlichen Screen
	 * @param screen : Screenobjekt
	 */
	public ApoSubGame(final ApoScreen screen) {
		super();
		this.screen = screen;
		this.screen.setSubGame(this);
	}
	
	public void init() {
		this.keyboard = new ApoKeyboard();
		this.screen.addKeyboard(this.keyboard);
		this.mouse = new ApoMouse();
		this.screen.addMouse(this.mouse);
		this.bShouldRepaint = true;
		this.bShouldThink = true;
		
		this.renderTime = 0;
		this.thinkTime = 0;
		
		this.load();
	}
	
	public void stopGame() {
		
	}

	/**
	 * gibt das Screenobjekt zurck
	 * @return gibt das Screenobjekt zurck
	 */
	public final ApoScreen getScreen() {
		return this.screen;
	}

	@Override
	public void run() {
		long unprocessedTimeThink = 0;
		long unprocessedTimeRender = 0;
		long msPerTickThink = (long)(1000.0 / (double)(ApoConstants.FPS_THINK));
		long msPerTickRender = (long)(1000.0 / (double)(ApoConstants.FPS_RENDER));
		this.frames = 0;
		long lastFpsTime = System.currentTimeMillis() - 1000;
		this.last = System.currentTimeMillis();
		this.isRunning = true;

		// eigentliche Spiellogikschleife
		while (this.isRunning) {
			long now = System.currentTimeMillis();
			long passedTime = now - this.last;
			// falls Zeit vergangen ist seit dem letzten Aufruf
			if (passedTime > 0) {
				unprocessedTimeThink += passedTime;
				unprocessedTimeRender += passedTime;
				// wenn nachgedacht werden soll dann denke nach
				while (unprocessedTimeThink >= msPerTickThink) {
					if (unprocessedTimeThink > 10 * msPerTickThink) {
						unprocessedTimeThink = 0;
					} else {
						unprocessedTimeThink -= msPerTickThink;
					}
					if (!this.bShouldThink) {
						unprocessedTimeThink = 0;
					} else {
						this.think(msPerTickThink);
					}
				}
				// wenn gezeichnet werden soll, dann zeichne neu
				if (unprocessedTimeRender >= msPerTickRender) {
					unprocessedTimeRender -= msPerTickRender;
					if (this.bShouldRepaint) {
						this.render();
					}
				}
				this.last = now;
			}
			// falls eine Sekunde vergangen ist, dann aktualisiere die FPSanzahl
			if (System.currentTimeMillis() - lastFpsTime > 1000) {
				if (!this.bShouldRepaint) {
					this.render();
				}
				lastFpsTime += 1000;
				this.screen.setFps(this.frames);
				this.frames = 0;
			}

			// leg den Thread fr 10 Millisekunden schlafen
			try {
				Thread.sleep(10);
			} catch (InterruptedException e) {
			}
		}
	}

	/**
	 * gibt die Klasse mit den Keyboardeingaben zurck
	 * @return gibt die Klasse mit den Keyboardeingaben zurck
	 */
	public final ApoKeyboard getKeyboard() {
		return this.keyboard;
	}
	
	/**
	 * gibt zurck, ob das Spiel immer neu geupdatet werden soll oder nicht
	 * @return TRUE Spiel soll immer neu geupdatet werden, ansonsten FALSE
	 */
	public final boolean shouldThink() {
		return this.bShouldThink;
	}

	/**
	 * setzt den Wert, ob ein Spiel immer geupdatet werden soll oder nicht auf den bergebenen
	 * @param shouldThink : TRUE Spiel soll immer neu geupdatet werden, ansonsten FALSE
	 */
	public void setShouldThink(boolean shouldThink) {
		this.bShouldThink = shouldThink;
	}

	/**
	 * gibt zurck, ob das Spiel immer neu gezeichnet werden soll oder nicht
	 * @return TRUE Spiel soll immer neu gezeichnet werden, FALSE Spiel soll nur neu gezeichnet werden, wenn man ber einen Button fhrt
	 */
	public final boolean shouldRepaint() {
		return this.bShouldRepaint;
	}

	/**
	 * setzt den Wert, ob neu gezeichnet werden soll auf den bergebenen
	 * @param shouldRepaint : TRUE Spiel soll immer neu gezeichnet werden, FALSE Spiel soll nur neu gezeichnet werden, wenn man ber einen Button fhrt
	 */
	public void setShouldRepaint(boolean shouldRepaint) {
		this.bShouldRepaint = shouldRepaint;
	}
	
	/**
	 * wird aufgerufen wenn nachgedacht werden soll
	 * und gibt zurck, ob berhaupt nachgedacht werden soll oder nicht
	 * @param delta : Zeit die seit dem letzten Aufruf vergangen ist in Millisekunden
	 * @return TRUE, Spiel wurde geupdatet ansonsten FALSE
	 */
	private final boolean think(long delta) {
		if (this.bShouldThink) {
			long time = System.nanoTime();
			this.update(delta);
			this.thinkTime = System.nanoTime() - time;
			return true;
		}
		return false;
	}

	/**
	 * wird aufgerufen, wenn das Spiel gezeichnet werden soll<br />
	 * Es unterscheidet dabei, ob es sich um eine normale Swingkomponente handelt, die einfach repaintet wird<br />
	 * oder um eine BufferStrategy, die anders gemalt wird<br />
	 */
	public void render() {
		if (this.screen.getBufferStrategy() != null) {
			long time = System.nanoTime();
			this.render(this.screen.getGraphics2D());
			this.screen.update();
			this.addFrame();
			this.renderTime = System.nanoTime() - time;
		} else {
			this.screen.repaint();
		}
	}

	/**
	 * addiert 1 auf die Frames
	 */
	public void addFrame() {
		this.frames += 1;
	}
	
	/**
	 * gibt zurck, wie lange das letzte Zeichnen gebraucht hat
	 * @return gibt zurck, wie lange das letzte Zeichnen gebraucht hat
	 */
	public final long getRenderTime() {
		return this.renderTime;
	}

	/**
	 * gibt zurck, wie lange das letzte "Nachdenken" gebraucht hat
	 * @return gibt zurck, wie lange das letzte "Nachdenken" gebraucht hat
	 */
	public final long getThinkTime() {
		return this.thinkTime;
	}

	/**
	 * Beendet das aktuelle Subgame und setzt die subgame ID auf den neuen Wert
	 * @param nextID : neue SubGame ID
	 */
	public void end(int nextID) {
		this.nextID = nextID;
		this.isRunning = false;
	}

	/**
	 * gibt die nchste ID wieder, wenn zwischen verschiedenen SubGames gewechselt werden soll
	 * @return gibt die nchste ID wieder, wenn zwischen verschiedenen SubGames gewechselt werden soll
	 */
	public final int getNextID() {
		return this.nextID;
	}

	/** Wird beim Initialisieren aufgerufen und ld alles wichtige */
	protected abstract void load();

	/**
	 * Methode wird alle delta Millisekunden aufgerufen und veranlasst das Spiel nachzudenken<br />
	 * die eigentliche Spiellogik steckt hier drin<br />
	 * @param delta : Zeit in Millisekunden, die seit dem letzten Aufruf vergangen ist
	 */
	protected abstract void update(long delta);

	/**
	 * Methode die aufgerufen wird, u, das das eigentliche Spiel auf den Bildschirm zu rendern
	 * @param g : Graphics2D-Objekt
	 */
	protected abstract void render(Graphics2D g);
}
