/*
 * Copyright (c) 2005-2013 Dirk Aporius <dirk.aporius@gmail.com>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.apogames.input;

import java.awt.event.KeyListener;
import java.awt.event.KeyEvent;
import java.util.ArrayList;
import java.util.TreeMap;

/**
 * Klasse, die sich um die Keyboardeingaben kmmert
 * @author Dirk Aporius
 *
 */
public final class ApoKeyboard implements KeyListener {

	/** TreeMap mit den Tasten, welches gerade gedrckt werden */
	private TreeMap<Integer, Boolean> keys;
	/** ArrayList mit den Tasten, welche gerade losgelassen wurden */
	private ArrayList<Integer> releasedKeys;
	/** ArrayList mit den Tasten, welche gerade losgelassen wurden */
	private ArrayList<Integer> releasedCharKeys;
	/** ArrayList mit den Tasten, welche gerade gedrckt werden */
	private ArrayList<Integer> pressedKeys;

	/**
	 * Konstruktor
	 */
	public ApoKeyboard() {
		this.keys = new TreeMap<Integer, Boolean>();
		this.releasedKeys = new ArrayList<Integer>();
		this.releasedCharKeys = new ArrayList<Integer>();
		this.pressedKeys = new ArrayList<Integer>();
	}

	/**
	 * gibt zurck, ob eine bergebene Taste derzeit gedrckt wird oder nicht
	 * @param key : Zu berprfende Taste, ob sie gedrckt wird
	 * @return TRUE, Taste wird gedrckt, else FALSE, Taste wird nicht gedrckt
	 */
	public boolean isPressed(int key) {
		if (this.keys.containsKey(key)) {
			return this.keys.get(key);
		} else {
			return false;
		}
	}

	/**
	 * gibt ein Array mit Tasten zurck, welche gerade losgelassen wurden, zurck
	 * @return gibt ein Array mit Tasten zurck, welche gerade losgelassen wurden, zurck
	 */
	public synchronized int[][] getReleased() {
		if ((this.releasedKeys == null) || (this.releasedKeys.size() <= 0)) {
			return new int[0][0];
		}
		int[][] released = new int[this.releasedKeys.size()][2];
		for (int i = 0; i < this.releasedKeys.size(); i++) {
			try {
				released[i][0] = this.releasedKeys.get(i);
				released[i][1] = this.releasedCharKeys.get(i);
			} catch (Exception ex) {
			}
		}
		this.releasedKeys.clear();
		this.releasedCharKeys.clear();
		return released;
	}

	/**
	 * gibt ein Array mit Tasten zurck, welche gerade gedrckt werden, zurck
	 * @return gibt ein Array mit Tasten zurck, welche gerade gedrckt werden, zurck
	 */
	public synchronized int[] getPressed() {
		if ((this.pressedKeys == null) || (this.pressedKeys.size() <= 0)) {
			return new int[0];
		}
		int[] pressed = new int[this.pressedKeys.size()];
		for (int i = 0; i < this.pressedKeys.size(); i++) {
			pressed[i] = this.pressedKeys.get(i);
		}
		return pressed;
	}

	public void keyPressed(KeyEvent e) {
		this.keys.put(e.getKeyCode(), true);
		if (this.pressedKeys.indexOf(e.getKeyCode()) < 0) {
			this.pressedKeys.add(e.getKeyCode());
		}
	}

	public void keyReleased(KeyEvent e) {
		this.keys.put(e.getKeyCode(), false);
		this.releasedCharKeys.add((int)e.getKeyChar());
		this.releasedKeys.add(e.getKeyCode());
		this.pressedKeys.remove((Integer) e.getKeyCode());
	}

	public void keyTyped(KeyEvent e) {
		this.keys.put(e.getKeyCode(), false);
		this.releasedCharKeys.add((int)e.getKeyChar());
		this.releasedKeys.add(e.getKeyCode());
		this.pressedKeys.remove((Integer) e.getKeyCode());
	}
}
